@@ -603,6 +603,9 @@ glade_command_set_property_description (GladeCommandSetProperty *me,
 			(GLADE_WIDGET_ADAPTOR (sdata->property->klass->handle),
 			 sdata->property->klass, sdata->new_value, fmt);
 
+		g_assert (sdata->property->klass->name);
+		g_assert (sdata->property->widget->name);
+
 		if (!value_name || strlen (value_name) > MAX_UNDO_MENU_ITEM_VALUE_LEN
 		    || strchr (value_name, '_')) {
 			description = g_strdup_printf (_("Setting %s of %s"),
@@ -611,7 +614,8 @@ glade_command_set_property_description (GladeCommandSetProperty *me,
 		} else {
 			description = g_strdup_printf (_("Setting %s of %s to %s"),
 						       sdata->property->klass->name,
-						       sdata->property->widget->name, value_name);
+						       sdata->property->widget->name,
+						       value_name);
 		}
 		g_free (value_name);
 	}
@@ -936,6 +940,9 @@ get_all_parentless_reffed_widgets (GList *reffed, GladeWidget *widget)
 	GList *children, *l, *list;
 	GladeWidget *child;
 
+	if ((list = glade_widget_get_parentless_reffed_widgets (widget)) != NULL)
+		reffed = g_list_concat (reffed, list);
+
 	children = glade_widget_adaptor_get_children (widget->adaptor,
 						      widget->object);
 
@@ -943,13 +950,12 @@ get_all_parentless_reffed_widgets (GList *reffed, GladeWidget *widget)
 	{
 		if ((child = glade_widget_get_from_gobject (l->data)) != NULL)
 		{
-			if ((list = glade_widget_get_parentless_reffed_widgets (child)) != NULL)
-				reffed = g_list_concat (reffed, list);
-
 			reffed = get_all_parentless_reffed_widgets (reffed, child);
 		}
 	}
 
+	g_list_free (children);
+
 	return reffed;
 }
 
@@ -992,11 +998,9 @@ glade_command_add (GList            *widgets,
 	 */
 	widget = GLADE_WIDGET (widgets->data);
 	if (placeholder && GTK_IS_WINDOW (widget->object) == FALSE)
-	{
-		GladeWidget *some_widget = glade_placeholder_get_parent (placeholder);
-		me->project = glade_widget_get_project (some_widget);
-	}
-	else me->project = glade_app_get_project();
+		me->project = glade_placeholder_get_project (placeholder);
+	else 
+		me->project = glade_app_get_project();
 
 	GLADE_COMMAND (me)->description = 
 		g_strdup_printf (_("Add %s"), g_list_length (widgets) == 1 ? 
@@ -1300,7 +1304,8 @@ glade_command_add_execute (GladeCommandAddRemove *me)
 					 * otherwise prioritize packing defaults. 
 					 */
 					saved_props =
-						glade_widget_dup_properties (cdata->widget->packing_properties, 
+						glade_widget_dup_properties (cdata->widget, 
+									     cdata->widget->packing_properties, 
 									     FALSE, FALSE, FALSE);
 					
 					glade_widget_set_packing_properties (cdata->widget, cdata->parent);
@@ -1383,6 +1388,7 @@ glade_command_add_execute (GladeCommandAddRemove *me)
 			/* Toplevels get pasted to the active project */
 			add_project = (me->from_clipboard && cdata->widget->parent == NULL) ?
 				active_project : me->project;
+
 			glade_project_add_object (add_project, cdata->project, cdata->widget->object);
 
 			for (l = cdata->reffed; l; l = l->next)
@@ -1395,7 +1401,7 @@ glade_command_add_execute (GladeCommandAddRemove *me)
 
 			glade_widget_show (cdata->widget);
 		}
-		glade_app_selection_changed ();
+		glade_app_queue_selection_changed ();
 	}
 	return TRUE;
 	
@@ -1412,14 +1418,6 @@ glade_command_remove_execute (GladeCommandAddRemove *me)
 	{
 		cdata = list->data;
 
-		if (cdata->parent)
-		{
-			if (cdata->placeholder)
-				glade_widget_replace(cdata->parent, cdata->widget->object, G_OBJECT (cdata->placeholder));
-			else
-				glade_widget_remove_child (cdata->parent, cdata->widget);
-		}
-
 		glade_project_remove_object(GLADE_PROJECT (cdata->widget->project), cdata->widget->object);
 		
 		for (l = cdata->reffed; l; l = l->next)
@@ -1428,6 +1426,14 @@ glade_command_remove_execute (GladeCommandAddRemove *me)
 			glade_project_remove_object(GLADE_PROJECT (cdata->widget->project), reffed->object);
 		}
 
+		if (cdata->parent)
+		{
+			if (cdata->placeholder)
+				glade_widget_replace(cdata->parent, cdata->widget->object, G_OBJECT (cdata->placeholder));
+			else
+				glade_widget_remove_child (cdata->parent, cdata->widget);
+		}
+
 		glade_widget_hide (cdata->widget);
 	}
 
@@ -1480,7 +1486,7 @@ glade_command_add_remove_finalize (GObject *obj)
 			if (cdata->handler_id)
 				g_signal_handler_disconnect (cdata->placeholder,
 							     cdata->handler_id);
-			if (GTK_OBJECT_FLOATING (cdata->placeholder))
+			if (g_object_is_floating (G_OBJECT (cdata->placeholder)))
 				gtk_widget_destroy (GTK_WIDGET (cdata->placeholder));
 		}
 
@@ -1729,12 +1735,11 @@ glade_command_create(GladeWidgetAdaptor *adaptor, GladeWidget *parent, GladePlac
 		return NULL;
 	}
 	widgets = g_list_prepend(widgets, widget);
-	glade_command_push_group(_("Create %s"), g_list_length (widgets) == 1 ? widget->name : _("multiple"));
+	glade_command_push_group(_("Create %s"), widget->name);
 	glade_command_add(widgets, parent, placeholder, FALSE);
 	glade_command_pop_group();
-	
-	if (widgets)
-		g_list_free(widgets);
+
+	g_list_free(widgets);
 	
 	return widget;
 }
@@ -1809,6 +1814,67 @@ glade_command_copy(GList *widgets)
 		g_list_free(copied_widgets);
 }
 
+#if 0
+static void
+glade_command_break_references_for_widget (GladeWidget *widget, GList *widgets)
+{
+	GList *l, *children;
+		
+	for (l = widget->properties; l; l = l->next)
+	{
+		property = l->data;
+		
+		if (glade_property_class_is_object (property->klass) &&
+		    property->klass->parentless_widget == FALSE)
+		{
+			GList   *obj_list;
+			GObject *reffed_obj = NULL;
+			GladeWidget *reffed_widget;
+			
+			if (GLADE_IS_PARAM_SPEC_OBJECTS (klass->pspec))
+			{
+				glade_property_get (property, &obj_list);
+				
+			}
+			else
+			{
+				glade_property_get (property, &reffed_obj);
+			}				
+		}
+	}
+
+	children = glade_widget_adaptor_get_children (widget->adaptor,
+						      widget->object);
+
+	for (l = children; l; l = l->next)
+	{
+		if ((child = glade_widget_get_from_gobject (l->data)) != NULL)
+			glade_command_break_references_for_widget (child, widgets);
+	}
+
+	g_list_free (children);
+}
+
+static void
+glade_command_break_references (GladeProject *project, GList *widgets)
+{
+	GList *list;
+	GladeWidget *widget;
+
+	for (list = widgets; list && list->data; list = list->next)
+	{
+		widget = l->data;
+
+		if (project == widget->project)
+			continue;
+
+		glade_command_break_references_for_widget (widget, widgets);
+	}
+
+
+}
+#endif
+
 /**
  * glade_command_paste:
  * @widgets: a #GList of #GladeWidget
@@ -1824,10 +1890,18 @@ glade_command_paste(GList *widgets, GladeWidget *parent, GladePlaceholder *place
 {
 	GList *list, *copied_widgets = NULL;
 	GladeWidget *copied_widget = NULL;
+/* 	GladeProject  *target_project; */
 	gboolean exact;
 	
 	g_return_if_fail (widgets != NULL);
 	
+/* 	if (placeholder && GTK_IS_WINDOW (widget->object) == FALSE) */
+/* 		target_project = glade_placeholder_get_project (placeholder); */
+/* 	else if (parent && GTK_IS_WINDOW (widget->object) == FALSE) */
+/* 		target_project = glade_widget_get_project (parent); */
+/* 	else  */
+/* 		target_project = glade_app_get_project(); */
+
 	for (list = widgets; list && list->data; list = list->next)
 	{
 		exact = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (list->data), "glade-command-was-cut"));
@@ -1835,7 +1909,14 @@ glade_command_paste(GList *widgets, GladeWidget *parent, GladePlaceholder *place
 		copied_widget = glade_widget_dup(list->data, exact);
 		copied_widgets = g_list_prepend(copied_widgets, copied_widget);
 	}
+
 	glade_command_push_group(_("Paste %s"), g_list_length (widgets) == 1 ? copied_widget->name : _("multiple"));
+
+	/* When pasting widgets across projects, we nullify the property references that
+	 * are not satisfied by the paste list.
+	 */
+
+
 	glade_command_add(copied_widgets, parent, placeholder, TRUE);
 	glade_command_pop_group();
 	
@@ -2483,7 +2564,6 @@ typedef struct {
 	GladeProject  *project;
 	GladeNamingPolicy policy;
 	GladeNamingPolicy old_policy;
-	gboolean run_once;
 } GladeCommandSetPolicy;
 
 
@@ -2505,14 +2585,12 @@ glade_command_set_policy_execute(GladeCommand *cmd)
 	g_return_val_if_fail (me->project != NULL, TRUE);
 
 	/* set the new policy */
-	glade_project_set_naming_policy (me->project, me->policy, me->run_once == FALSE);
+	glade_project_set_naming_policy (me->project, me->policy);
 
 	/* swap the current values with the old values to prepare for undo */
 	policy         = me->policy;
 	me->policy     = me->old_policy;
 	me->old_policy = policy;
-
-	me->run_once = TRUE;
 	
 	return TRUE;
 }
@@ -2587,8 +2665,6 @@ glade_command_set_project_naming_policy  (GladeProject       *project,
 		me->policy = policy;
 		me->old_policy = glade_project_get_naming_policy (project);
 
-		me->run_once = FALSE;
-
 		GLADE_COMMAND(me)->description = g_strdup_printf("dummy string");
 	
 		glade_command_check_group(GLADE_COMMAND(me));
