@@ -23,7 +23,7 @@ typedef struct _GladeWidgetClass  GladeWidgetClass;
 
 struct _GladeWidget
 {
-	GObject parent_instance;
+	GInitiallyUnowned parent_instance;
 
 	GladeWidgetAdaptor *adaptor; /* An adaptor class for the object type */
 
@@ -117,7 +117,7 @@ struct _GladeWidget
 
 struct _GladeWidgetClass
 {
-	GObjectClass parent_class;
+	GInitiallyUnownedClass parent_class;
 
 	void         (*add_child)               (GladeWidget *, GladeWidget *, gboolean);
 	void         (*remove_child)            (GladeWidget *, GladeWidget *);
@@ -273,6 +273,10 @@ GladeEditorProperty    *glade_widget_create_editor_property (GladeWidget      *w
 							     gboolean          use_command);
 
 gchar                  *glade_widget_generate_path_name     (GladeWidget      *widget);
+
+gboolean                glade_widget_is_ancestor            (GladeWidget      *widget,
+							     GladeWidget      *ancestor);
+
 /*******************************************************************************
                       Project, object property references
  *******************************************************************************/
